import { jsx } from 'react/jsx-runtime';
import { useState, useCallback, useEffect, useRef } from 'react';
import styled from '@emotion/styled';

const useAudioContext = (audioUrl, useMicrophone, fftSize = 2048, smoothingTimeConstant = 0.8, minDecibels = -90, maxDecibels = -10) => {
    const [audioState, setAudioState] = useState({
        audioContext: null,
        analyser: null,
        dataArray: null,
        source: null,
    });
    const initializeAudioContext = useCallback(async () => {
        try {
            const audioContext = new (window.AudioContext || window.webkitAudioContext)();
            const analyser = audioContext.createAnalyser();
            analyser.fftSize = fftSize;
            analyser.smoothingTimeConstant = smoothingTimeConstant;
            analyser.minDecibels = minDecibels;
            analyser.maxDecibels = maxDecibels;
            const bufferLength = analyser.frequencyBinCount;
            const dataArray = new Uint8Array(bufferLength);
            if (useMicrophone) {
                const stream = await navigator.mediaDevices.getUserMedia({ audio: true });
                const source = audioContext.createMediaStreamSource(stream);
                source.connect(analyser);
                setAudioState({
                    audioContext,
                    analyser,
                    dataArray,
                    source,
                });
            }
            else if (audioUrl) {
                const response = await fetch(audioUrl);
                const arrayBuffer = await response.arrayBuffer();
                const audioBuffer = await audioContext.decodeAudioData(arrayBuffer);
                const source = audioContext.createBufferSource();
                source.buffer = audioBuffer;
                source.connect(analyser);
                analyser.connect(audioContext.destination);
                source.start(0);
                setAudioState({
                    audioContext,
                    analyser,
                    dataArray,
                    source,
                });
            }
        }
        catch (error) {
            console.error('Error initializing audio context:', error);
        }
    }, [audioUrl, useMicrophone, fftSize, smoothingTimeConstant, minDecibels, maxDecibels]);
    const cleanup = useCallback(() => {
        if (audioState.source) {
            audioState.source.disconnect();
        }
        if (audioState.analyser) {
            audioState.analyser.disconnect();
        }
        if (audioState.audioContext) {
            audioState.audioContext.close();
        }
    }, [audioState]);
    useEffect(() => {
        initializeAudioContext();
        return cleanup;
    }, [initializeAudioContext, cleanup]);
    return audioState;
};

const Canvas$2 = styled.canvas `
  width: 100%;
  height: 100%;
`;
const Container$2 = styled.div `
  position: relative;
  width: 100%;
  height: 100%;
  background-color: ${props => props.backgroundColor};
`;
const WaveformVisualizer = ({ audioUrl, useMicrophone, width = 800, height = 200, backgroundColor = '#1a1a2e', foregroundColor = '#4CAF50', barWidth = 3, barSpacing = 1, barRadius = 0, smoothingTimeConstant = 0.8, fftSize = 2048, minDecibels = -90, maxDecibels = -10, }) => {
    const canvasRef = useRef(null);
    const audioState = useAudioContext(audioUrl, useMicrophone, fftSize, smoothingTimeConstant, minDecibels, maxDecibels);
    const animationFrameId = useRef(null);
    const draw = () => {
        if (!canvasRef.current || !audioState.analyser || !audioState.dataArray)
            return;
        const canvas = canvasRef.current;
        const ctx = canvas.getContext('2d');
        if (!ctx)
            return;
        const bufferLength = audioState.analyser.frequencyBinCount;
        const dataArray = new Uint8Array(bufferLength);
        audioState.analyser.getByteTimeDomainData(dataArray);
        ctx.fillStyle = backgroundColor;
        ctx.fillRect(0, 0, width, height);
        ctx.lineWidth = barWidth;
        ctx.strokeStyle = foregroundColor;
        ctx.beginPath();
        const sliceWidth = (width * 1.0) / bufferLength;
        let x = 0;
        for (let i = 0; i < bufferLength; i++) {
            const v = dataArray[i] / 128.0;
            const y = (v * height) / 2;
            if (i === 0) {
                ctx.moveTo(x, y);
            }
            else {
                ctx.lineTo(x, y);
            }
            x += sliceWidth;
        }
        ctx.lineTo(width, height / 2);
        ctx.stroke();
        animationFrameId.current = requestAnimationFrame(draw);
    };
    useEffect(() => {
        if (canvasRef.current) {
            canvasRef.current.width = width;
            canvasRef.current.height = height;
        }
        draw();
        return () => {
            if (animationFrameId.current) {
                cancelAnimationFrame(animationFrameId.current);
            }
        };
    }, [audioState]);
    return (jsx(Container$2, { backgroundColor: backgroundColor, children: jsx(Canvas$2, { ref: canvasRef }) }));
};

const Canvas$1 = styled.canvas `
  width: 100%;
  height: 100%;
`;
const Container$1 = styled.div `
  position: relative;
  width: 100%;
  height: 100%;
  background-color: ${props => props.backgroundColor};
`;
const FrequencyVisualizer = ({ audioUrl, useMicrophone, width = 800, height = 200, backgroundColor = '#1a1a2e', foregroundColor = '#4CAF50', gradientColors = ['#00bcd4', '#4CAF50', '#8BC34A'], barWidth = 6, barSpacing = 2, barRadius = 4, smoothingTimeConstant = 0.8, fftSize = 2048, minDecibels = -90, maxDecibels = -10, }) => {
    const canvasRef = useRef(null);
    const audioState = useAudioContext(audioUrl, useMicrophone, fftSize, smoothingTimeConstant, minDecibels, maxDecibels);
    const animationFrameId = useRef(null);
    const createGradient = (ctx) => {
        if (!gradientColors || gradientColors.length < 2)
            return foregroundColor;
        const gradient = ctx.createLinearGradient(0, height, 0, 0);
        const step = 1 / (gradientColors.length - 1);
        gradientColors.forEach((color, index) => {
            gradient.addColorStop(index * step, color);
        });
        return gradient;
    };
    const draw = () => {
        if (!canvasRef.current || !audioState.analyser || !audioState.dataArray)
            return;
        const canvas = canvasRef.current;
        const ctx = canvas.getContext('2d');
        if (!ctx)
            return;
        const bufferLength = audioState.analyser.frequencyBinCount;
        const dataArray = new Uint8Array(bufferLength);
        audioState.analyser.getByteFrequencyData(dataArray);
        ctx.fillStyle = backgroundColor;
        ctx.fillRect(0, 0, width, height);
        const barCount = Math.min(bufferLength, Math.floor(width / (barWidth + barSpacing)));
        const totalWidth = barCount * (barWidth + barSpacing);
        const startX = (width - totalWidth) / 2;
        ctx.fillStyle = createGradient(ctx);
        for (let i = 0; i < barCount; i++) {
            const barHeight = (dataArray[i] / 255.0) * height;
            const x = startX + i * (barWidth + barSpacing);
            const y = height - barHeight;
            if (barRadius > 0) {
                ctx.beginPath();
                ctx.roundRect(x, y, barWidth, barHeight, barRadius);
                ctx.fill();
            }
            else {
                ctx.fillRect(x, y, barWidth, barHeight);
            }
        }
        animationFrameId.current = requestAnimationFrame(draw);
    };
    useEffect(() => {
        if (canvasRef.current) {
            canvasRef.current.width = width;
            canvasRef.current.height = height;
        }
        draw();
        return () => {
            if (animationFrameId.current) {
                cancelAnimationFrame(animationFrameId.current);
            }
        };
    }, [audioState]);
    return (jsx(Container$1, { backgroundColor: backgroundColor, children: jsx(Canvas$1, { ref: canvasRef }) }));
};

const Canvas = styled.canvas `
  width: 100%;
  height: 100%;
`;
const Container = styled.div `
  position: relative;
  width: 100%;
  height: 100%;
  background-color: ${props => props.backgroundColor};
`;
const CircularVisualizer = ({ audioUrl, useMicrophone, width = 800, height = 800, backgroundColor = '#1a1a2e', foregroundColor = '#4CAF50', gradientColors = ['#00bcd4', '#4CAF50', '#8BC34A'], barWidth = 3, barSpacing = 1, smoothingTimeConstant = 0.8, fftSize = 2048, minDecibels = -90, maxDecibels = -10, animationSpeed = 1, }) => {
    const canvasRef = useRef(null);
    const audioState = useAudioContext(audioUrl, useMicrophone, fftSize, smoothingTimeConstant, minDecibels, maxDecibels);
    const animationFrameId = useRef(null);
    const rotation = useRef(0);
    const createGradient = (ctx, radius) => {
        if (!gradientColors || gradientColors.length < 2)
            return foregroundColor;
        const gradient = ctx.createRadialGradient(width / 2, height / 2, 0, width / 2, height / 2, radius);
        gradientColors.forEach((color, index) => {
            gradient.addColorStop(index / (gradientColors.length - 1), color);
        });
        return gradient;
    };
    const draw = () => {
        if (!canvasRef.current || !audioState.analyser || !audioState.dataArray)
            return;
        const canvas = canvasRef.current;
        const ctx = canvas.getContext('2d');
        if (!ctx)
            return;
        const bufferLength = audioState.analyser.frequencyBinCount;
        const dataArray = new Uint8Array(bufferLength);
        audioState.analyser.getByteFrequencyData(dataArray);
        ctx.fillStyle = backgroundColor;
        ctx.fillRect(0, 0, width, height);
        const centerX = width / 2;
        const centerY = height / 2;
        const radius = Math.min(width, height) / 3;
        const bars = Math.min(bufferLength, 360);
        const angleStep = (2 * Math.PI) / bars;
        rotation.current += 0.002 * animationSpeed;
        if (rotation.current >= 2 * Math.PI) {
            rotation.current = 0;
        }
        ctx.save();
        ctx.translate(centerX, centerY);
        ctx.rotate(rotation.current);
        for (let i = 0; i < bars; i++) {
            const angle = i * angleStep;
            const magnitude = dataArray[i] / 255.0;
            const barHeight = magnitude * radius;
            const x1 = Math.cos(angle) * radius;
            const y1 = Math.sin(angle) * radius;
            const x2 = Math.cos(angle) * (radius + barHeight);
            const y2 = Math.sin(angle) * (radius + barHeight);
            ctx.strokeStyle = createGradient(ctx, radius + barHeight);
            ctx.lineWidth = barWidth;
            ctx.beginPath();
            ctx.moveTo(x1, y1);
            ctx.lineTo(x2, y2);
            ctx.stroke();
        }
        ctx.restore();
        animationFrameId.current = requestAnimationFrame(draw);
    };
    useEffect(() => {
        if (canvasRef.current) {
            canvasRef.current.width = width;
            canvasRef.current.height = height;
        }
        draw();
        return () => {
            if (animationFrameId.current) {
                cancelAnimationFrame(animationFrameId.current);
            }
        };
    }, [audioState]);
    return (jsx(Container, { backgroundColor: backgroundColor, children: jsx(Canvas, { ref: canvasRef }) }));
};

export { CircularVisualizer, FrequencyVisualizer, WaveformVisualizer, useAudioContext };
//# sourceMappingURL=index.esm.js.map
